"""Example 8 node directed graph from Robert Tarjan.

Reference:
[Figure 3] Tarjan, R. (1972). Depth-First Search and Linear Graph Algorithms.
SIAM Journal on Computing, 1(2), 146–160. https://doi.org/10.1137/0201010
"""

edge_weights = ["adjacency"]
